// file: isip_verify.cc
// version: $Id: isip_verify.cc,v 1.8 2003/09/30 02:55:22 parihar Exp $
//

// isip include files
//
#include <CommandLine.h>
#include <VectorFloat.h>

// main program starts here
//
int main (int argc, const char **argv) {

  // create command line
  //
  CommandLine cmdl;
    
  // declare variables to handle command line arguments
  //
  Filename lfile;
  String thresh_str;
  DebugLevel verbosity;
  String fn_spk;
  String fn_imp;

  // declare utterance processing variables
  //
  File file_spk;
  File file_imp;
  float thresh_rej = -1e07;
  float thresh_diff = 0.0;  
  String id;
  float z0, z1;
  boolean acpt;
  
  // declare miscellaneous local variables
  //
  String output;
  String tmp;

  //---------------------------------------------------------------------------
  //
  // configure and parse the command line:
  //  add only the options that are not included by default.
  // 
  //---------------------------------------------------------------------------

  // set up messages
  //
  cmdl.setUsage(
#include "usage_message.text"
                );
  cmdl.setHelp(
#include "help_message.text"
               );
  
  cmdl.setIdent("$Revision: 1.8 $",
		"$Name:  $",
		"$Date: 2003/09/30 02:55:22 $");
  
  // set command line options
  //
  cmdl.addOptionParam(lfile, CommandLine::DEF_PARAM_LFILE,
		      CommandLine::DEF_LFILE_NAME);
  cmdl.addOptionParam(thresh_str, L"thresholds", L"-1e07,0.0");
  cmdl.addOptionParam(verbosity, CommandLine::DEF_PARAM_VERBOSITY);
  
  // parse the command line
  //
  cmdl.parse(argc, argv);

  // check the validity of the command line arguments:
  //  - we require two filenames as arguments
  //  - we require one threshold parameter
  //  
  VectorFloat thresh(thresh_str);
  if ((cmdl.numArguments() != 2) ||
      (thresh.length() != 2)) {
    cmdl.printUsage();
  }

  // obtain the arguments
  //
  thresh_rej = (float)thresh(0);
  thresh_diff = (float)thresh(1);
  cmdl.getArgument(fn_spk, 0);
  cmdl.getArgument(fn_imp, 1);
 
  // configure the console
  //
  if (lfile.ne(CommandLine::DEF_LFILE_NAME)) {
    Console::open(lfile);
  }
  
  // echo the command line
  //
  if (verbosity > Integral::BRIEF) {
    cmdl.printCommandLine();
    cmdl.printVersion();
  }

  //--------------------------------------------------------------------------
  //
  // prepare the log files
  //
  //---------------------------------------------------------------------------

  if (verbosity > Integral::BRIEF) {
    Console::putNoWrap(L"preparing log files...");
  }
  output.assign(thresh(0), L"rejection threshold = %12.2f");
  Console::putNoWrap(output);
  output.assign(thresh(1), L"ratio threshold = %12.2f\n\n");
  Console::putNoWrap(output);
  Console::putNoWrap(L"[id]\t\t[z1]\t\t[z0]\t\t[diff]\t\t[accept]");

  // open the log files
  //
  if (verbosity > Integral::DETAILED) {
    Console::putNoWrap(L"opening speaker file...");
  }
  if (!file_spk.open(fn_spk, File::READ_ONLY)) {
    Console::put(L"Error in opening the speaker log file");
    cmdl.printUsage();
  }

  if (verbosity > Integral::DETAILED) {
    Console::putNoWrap(L"opening imposter file...");
  }
  if (!file_imp.open(fn_imp, File::READ_ONLY)) {
    Console::put(L"Error in opening the imposter log file");
    file_spk.close();
    cmdl.printUsage();
  }

  //--------------------------------------------------------------------------
  //
  // main processing loop
  //
  //---------------------------------------------------------------------------
  
  // increase the indention
  //
  if (verbosity > Integral::BRIEF) {
    Console::putNoWrap(L"beginning main processing loop...");
  }
  if (verbosity >= Integral::BRIEF) {
    Console::increaseIndention();
  }

  // loop over all entries in the speaker file and fetch speaker ids
  //
  long num_accepted = 0;
  long num_processed = 0;
  long pos10 = Integral::NO_POS;
  long pos20 = Integral::NO_POS;

  for (;;) {

    // search for the utterance id
    //
    long pos11 = file_spk.firstStr(L"processing file", pos10);
    if (pos11 == Integral::NO_POS) {
      if (verbosity > Integral::BRIEF) {
	Console::put(L"End Of File is reached");
      }
      break;
    }

    long pos12 = file_spk.firstStr(L"(", pos11);
    long pos13 = file_spk.firstStr(L"):", pos12);
    file_spk.substr(id, pos12+1, pos13-pos12-1);

    // echo the id
    //
    num_processed++;
    if (verbosity > Integral::BRIEF) {
      output.assign(L"processing utterance no. ");
      tmp.assign(num_processed, L"%ld");
      output.concat(tmp);
      Console::putNoWrap(output);
    }

    // search for the likelihood score against the speaker model
    //
    long pos14 = file_spk.firstStr(L"score:", pos13);
    long pos15 = file_spk.firstStr(L"-", pos14);
    long pos16 = file_spk.firstStr(L" ", pos15);
    file_spk.substr(tmp, pos15, pos16-pos15);
    tmp.get(z1);

    // prepare the start position for the next search,
    // search for the likelihood score against the imposter model
    // prepare the start position for the next search
    //
    pos10 = pos16;
    long pos24 = file_imp.firstStr(L"score:", pos20);
    long pos25 = file_imp.firstStr(L"-", pos24);
    long pos26 = file_imp.firstStr(L" ", pos25);
    file_imp.substr(tmp, pos25, pos26-pos25);
    tmp.get(z0);
    pos20 = pos26;

    // apply a decision rule:
    //  use a simple rejection / ratio test
    //
    if ((z1 > thresh_rej) && ((z1 - z0) > thresh_diff)) {
      acpt = true;
      num_accepted++;
    }
    else {
      acpt = false;
    }
    
    // output the results
    //
    output.assign(id);
    if (verbosity == Integral::NONE) {
      if (id.length() > 7) {
	output.concat(L"\t");      
      }
      else {
	output.concat(L"\t\t");
      }
    }
    else {
      if (id.length() > 5) {
	output.concat(L"\t");      
      }
      else {
	output.concat(L"\t\t");
      }
    }

    tmp.assign((float)z1, L"%7.2f");
    output.concat(tmp);
    output.concat(L"\t");
    tmp.assign((float)z0, L"%7.2f");
    output.concat(tmp);
    output.concat(L"\t");
    tmp.assign((float)(z1/z0), L"%2.7f");
    output.concat(tmp);
    output.concat(L"\t");

    if (acpt) {
      output.concat(L"yes");
    }
    else {
      output.concat(L"no");
    }
    output.concat(L"\n");
    Console::putNoWrap(output);
  }

  // close the input/output files
  //
  file_spk.close();
  file_imp.close();
  
  // decrease the indention and report results
  //
  if (verbosity >= Integral::BRIEF) {
    Console::decreaseIndention();
    output.assign(L"summary: ");
    tmp.assign(num_accepted, L"%ld");
    output.concat(tmp);
    output.concat(L" utterances accepted out of ");
    tmp.assign(num_processed, L"%ld");
    output.concat(tmp);
    output.concat(L" utterances processed");
    Console::putNoWrap(output);
    cmdl.printFinalReport(num_processed, num_processed);
  }
  
  // exit gracefully
  //
  Integral::exit();
}
