# file: vv/scripts/perl/vv_subroutines/vv_subs.pm
#
# this file contains the subroutines that pertain to running experiments with 
# the production system. Typically these will be wrappers around the production
# system functions and isip_verify utility.  Note that all functions in this
# package return 0 if there is an error and 1 otherwise. This is chosen to 
# coincide with the notion of true and false. Note that this is different than
# the return values for system calls where 1 is an error and 0 is no error
#

# system libraries
#
use FileHandle;
autoflush STDOUT;

# global variables
#

# define the utilities that will be needed for this script
#
$ISIP_RECOGNIZE = "/isip/d222/tang/tools/bin/i386-pc-solaris2.8//isip_recognize";
$ISIP_VERIFY = "/isip/data/research/voice_analysis/exp_005/isip_vv_1.0/isip_verify/isip_verify.exe";

#----------------------------------------------------------------------------
#
# wrappers around utilities
#
#----------------------------------------------------------------------------

# subroutine: run_isip_recognize
#
# arguments:
#  @_ : ($param_file, $utt_ids_file, $log_file, $debug_level, $verbosity)
#
# description: this routine runs isip_recognize utility
#
# return: 0 if error 1 if no error
#
sub run_isip_recognize ($$$$$) {

    # get parameters from arg list
    #
    my $param_file = shift @_;
    my $utt_ids_file = shift @_;
    my $log_file = shift @_;
    my $debug_level = shift @_;
    my $verbosity = shift @_;

    if (($param_file eq "") || ($utt_ids_file eq "") || ($log_file eq "")
	|| ($debug_level eq "") || ($verbosity eq ""))  {
	print "ERROR: run_isip_recognize - bad argument list\n";
	return 0;
    }
    
    if ($verbosity eq "NONE") {
	$verbosity = "BRIEF";
    }
    
    # build the argument list for the call to isip_recognize
    #
    @args = ($ISIP_RECOGNIZE, "-param", $param_file, "-list", $utt_ids_file,
	     "-verbose", $verbosity, "-log_file" , $log_file, 
	     "-debug", $debug_level);
    $out = join(" ", @args);
    print "COMMAND: $out\n";
    
    # run isip_recognize
    #
    if (system(@args) != 0) {
	print "ERROR: run_isip_recognize - call to isip_recognize failed\n";
	return 0;
    }

    # exit gracefully
    #
    return 1;
}

# subroutine: run_isip_verify
#
# arguments:
#  @_ : ($threshold, $spk_log_file, $imp_log_file, $output_file)
#
# description: this routine runs isip_verify utility
#
# return: 0 if error 1 if no error
#
sub run_isip_verify ($$$$$) {

    # get parameters from arg list
    #
    my $threshold = shift @_;
    my $spk_log_file = shift @_;
    my $imp_log_file = shift @_;
    my $output_file = shift @_;
    my $verbosity = shift @_;

    if (($spk_log_file eq "") || ($imp_log_file eq "") 
	|| ($verbosity eq ""))  {
	print "ERROR: run_isip_verify - bad argument list\n";
	return 0;
    }

    # build the argument list for the call to isip_recognize
    #
    @args = ($ISIP_VERIFY, "-verbosity", $verbosity, 
	     "-thresholds", " $threshold", "-log_file", $output_file, 
	     $spk_log_file, $imp_log_file);
    $out = join(" ", @args);
    print "COMMAND: $out\n";
    
    # run isip_verify
    #
    if (system(@args) != 0) {
	print "ERROR: run_isip_verify - call to isip_verify failed\n";
	return 0;
    }

    # exit gracefully
    #
    return 1;
}

# subroutine: isip_delete
#
# arguments:
#  @_ : (file to be deleted)
#
# description: this routine attempts to delete a given file
#
# return: 0 if error 1 if no error
#
sub isip_delete ($) {

    # parse the argument list
    #
    my $delete_file = shift @_;

    if ($delete_file eq "") {
	print "ERROR: isip_delete - bad argument list\n";
	return 0;
    }

    # delete the file
    #
    if (-e $delete_file) {
	@file_list = ("$delete_file");
	@successes = unlink @file_list;
	if ($#successes < 0) {
	    print "ERROR: isip_delete - could not delete file:\n";
	    print "ERROR: $delete_file\n";
	    return 0;
	}
    }

    # exit gracefully
    #
    return 1;
}
