#!/usr/local/bin/perl
#
# file: vv_exp/isip_vv.pl.in
#
$ISIP_HELP_FILE = <<__ISIP_HELP_HERE_FILE__;
name: isip_vv
synopsis: 
    isip_vv -thresholds <thresholds> -speaker <speaker_parameter_file> \
    -imposter <imposter_parameter_file> -output <results_file> \
    -debug <debug_level> -verbosity <verbosity> <utterance_ids_file>

descr: 
    This program runs a speaker verification experiment given the trained 
    models.

example:
    isip_vv -thresholds "-1e07,0" -speaker spk_params.sof \
    -imposter imp_params.sof -output results.text -debug BRIEF \
    -verbosity BRIEF utterance_ids.sof

options: 

   -thresholds <thresholds> : 
      the decision thresholds (default is "-1e07,0")

   -speaker <speaker_parameter_file> : 
      specifies the parameter file in sof format for isip_recognize utility 
      to score the input utterances on the claimed speaker model.

   -imposter <imposter_parameter_file> : 
      specifies the parameter file in sof format for isip_recognize utility 
      to score the input utterances on the imposter model.

   -output <results_file> : 
      specifies the output file where the results are written 
      (default is stdout).

   -debug <debug_level>
      specifies the debug levels specified in the isip_recognize utility.
      Choices are:
      NONE (default)
      BRIEF
      DETAILED
      ALL

   -verbosity <verbosity>
      specifies the verbosity specified in the isip_recognize and 
      isip_verify utilities.
      Choices are:
      NONE (default)
      BRIEF
      DETAILED
      ALL

   -clean :  
      if this flag is thrown then all previous experiments run will be cleaned
      up (i.e. deleted). All the intermediate and output files in the 
      experiment directory will be deleted. This flag is used to start from a
      fresh state.

arguments:

   utterance_ids_file : Identifiers for the input utterances in sof sdb format
 
man page: none
__ISIP_HELP_HERE_FILE__

# define the path to this script
#
$VV_PATH = "/isip/d222/research/voice_analysis/exp_005/isip_vv_1.0/isip_vv";

# system packages
#
use File::Basename;
use File::Path;
use File::Copy;
use FileHandle;
autoflush STDOUT;

# isip libraries
#
require "$VV_PATH/lib/scripts/perl/command_line.pm";
require "$VV_PATH/lib/scripts/perl/vv_subs.pm";

#----------------------------------------------------------------------------
#
# constant and global variable definitions
#
#----------------------------------------------------------------------------

# define the program name
#
@tmp = split(/\//, $0);
$ISIP_PROG = pop(@tmp);

# define a separator for output comments
#
$VV_LINE_SEP = "-------------------  $ISIP_PROG  ----------------";

# define files and locations to be used by this script
#
$VV_PATH_TO_EXP = "$VV_PATH/exp";
$VV_SPK_LOG_FILE = "$VV_PATH_TO_EXP/speaker_log.text";
$VV_IMP_LOG_FILE = "$VV_PATH_TO_EXP/imposter_log.text";

# define the sub-scripts called by this script
#
$VV_PATH_TO_SCRIPTS = "$VV_PATH/bin/scripts";

# define command-line options
#
$VV_OPT_THRESHOLDS = "-thresholds";
$VV_OPT_SPK_PARAM_FILE = "-speaker";
$VV_OPT_IMP_PARAM_FILE = "-imposter";
$VV_OPT_OUTPUT_FILE = "-output";
$VV_OPT_DEBUG_LEVEL = "-debug";
$VV_OPT_VERBOSITY = "-verbosity";
$VV_OPT_CLEAN = "-clean";

$VV_DEBUG_LEVEL_NONE = "NONE";
$VV_DEBUG_LEVEL_BRIEF = "BRIEF";
$VV_DEBUG_LEVEL_DETAILED = "DETAILED";
$VV_DEBUG_LEVEL_ALL = "ALL";
@VV_DEBUG_LEVEL_TYPES = ($VV_DEBUG_LEVEL_NONE, $VV_DEBUG_LEVEL_BRIEF,
			 $VV_DEBUG_LEVEL_DETAILED, $VV_DEBUG_LEVEL_ALL);

#define the defaults
#
$VV_THRESHOLDS_DEF = "-1e07,0";
$VV_DEBUG_LEVEL_DEF = $VV_DEBUG_LEVEL_NONE;

#----------------------------------------------------------------------------
#
# command line processing
#
#----------------------------------------------------------------------------
# print a debugging message
#
print "$VV_LINE_SEP\n";
print "INFO: isip_vv - checking command-line arguments\n";

# parse the command line
#
my ($thresholds, $spk_par_file, $imp_par_file, $output_file, $debug_level, 
    $verbosity, $clean, $utt_ids_file) =  
    command_line(1, $VV_OPT_THRESHOLDS, 1, $VV_OPT_SPK_PARAM_FILE, 1,
		 $VV_OPT_IMP_PARAM_FILE, 1, $VV_OPT_OUTPUT_FILE, 1,
		 $VV_OPT_DEBUG_LEVEL, 1, $VV_OPT_VERBOSITY, 1,
		 $VV_OPT_CLEAN, 0);

# set defaults if necessary
#
# clean up if desired
#
if ($clean ne "") {
    (run_cleanup() == 1) or isip_die("ERROR: cleanup failed...exiting");
    print "INFO: isip_vv - cleanup succeeded...exiting\n";
    exit 0;
}

if ($thresholds eq "") {
    $thresholds = $VV_THRESHOLDS_DEF;
}

if ($debug_level eq "") {
    $debug_level = $VV_DEBUG_LEVEL_DEF;
}

if ($verbosity eq "") {
    $verbosity = $VV_DEBUG_LEVEL_DEF;
}

# verify debug options
#
$debug_level_v = grep /\b$debug_level\b/, @VV_DEBUG_LEVEL_TYPES;

if ($debug_level_v < 1) {
    $valid_debug_levels = join(", ", @VV_DEBUG_LEVEL_TYPES);
    isip_die ("ERROR: $debug_level is not a valid debug type\n ERROR: valid debug levels are $valid_debug_levels");
}

# verify verbosity
#
$verbosity_v = grep /\b$verbosity\b/, @VV_DEBUG_LEVEL_TYPES;

if ($verbosity_v < 1) {
    $valid_verbosity = join(", ", @VV_DEBUG_LEVEL_TYPES);
    isip_die ("ERROR: $verbosity is not a valid debug type\n ERROR: valid debug levels are $valid_verbosity");
}

# verify speaker and imposter parmeter files
#
if ($spk_par_file eq "") {
    isip_die ("ERROR: parameter file to score input utterances on speaker model  not provided\n");
}
if ($imp_par_file eq "") {
    isip_die ("ERROR: parameter file to score input utterances on imposter model  not provided\n");
}

print "INFO: isip_vv - running with parameters: \n";
$out_line = join (" ", $VV_OPT_THRESHOLDS, $thresholds,
		  $VV_OPT_SPK_PARAM_FILE, $spk_par_file,
		  $VV_OPT_IMP_PARAM_FILE, $imp_par_file,
		  $VV_OPT_OUTPUT_FILE, $output_file,
		  $VV_OPT_DEBUG_LEVEL, $debug_level,
		  $VV_OPT_VERBOSITY, $verbosity,
		  $utt_ids_file);

print "$out_line\n";

# create directory structure for the output lists and transcriptions
#
if (! -d "$VV_PATH_TO_EXP") {
    (isip_mkdir($VV_PATH_TO_EXP) == 1)
	or isip_die ("ERROR: error creating directories...exiting");
}

#----------------------------------------------------------------------------
#
# run the isip_recognize utility to generate score of the input utterances on
# the speaker model
#
#----------------------------------------------------------------------------

print "INFO: isip_vv - generating the scores on the speaker models\n";

# delete the log file
#
if ( -e "$VV_SPK_LOG_FILE") {
    (isip_delete($VV_SPK_LOG_FILE) == 1)
	or isip_die ("ERROR: deleting log file...exiting");
}

# run the isip-recognize utility
#
(run_isip_recognize($spk_par_file, $utt_ids_file, $VV_SPK_LOG_FILE, 
		    $debug_level, $verbosity)  == 1) 
    or isip_die ("scoring on claimed speaker model failed...exiting");

print "$VV_LINE_SEP\n";

#----------------------------------------------------------------------------
#
# run the isip_recognize utility to generate score of the input utterances on
# the imposter model
#
#----------------------------------------------------------------------------

print "INFO: isip_vv - generating the scores on the imposter models\n";

# delete the log file
#
if ( -e "$VV_IMP_LOG_FILE") {
    (isip_delete($VV_IMP_LOG_FILE) == 1)
	or isip_die ("ERROR: deleting log file...exiting");
}

# run the isip-recognize utility
#
(run_isip_recognize($imp_par_file, $utt_ids_file, $VV_IMP_LOG_FILE, 
		    $debug_level, $verbosity)  == 1) 
    or isip_die ("scoring on imposter model failed...exiting");

print "$VV_LINE_SEP\n";

#----------------------------------------------------------------------------
#
# run the isip_verify utlity to make a decision for the input utterances
#
#----------------------------------------------------------------------------

print "INFO: isip_vv - making decision on the verification of the input utterances\n";

# run the isip_verify utility
#
(run_isip_verify($thresholds, $VV_SPK_LOG_FILE, $VV_IMP_LOG_FILE,
		 $output_file, $verbosity)  == 1) 
    or isip_die ("decision making failed ...exiting");

print "$VV_LINE_SEP\n";

# as usual exit gracefully
#
#return 1;

#----------------------------------------------------------------------------
#
# subroutines
#
#----------------------------------------------------------------------------

# subroutine: run_cleanup
#
# arguments: none
#
# description: this routine cleans up the experiment directory
#
# return: 1 if error, 0 if no error
#
sub run_cleanup() {

    print "INFO: run_cleanup - cleaning up old experiments:\n";

    # run the command
    #
    if (rmtree($VV_PATH_TO_EXP) < 0) {
	print "ERROR: run_cleanup - could not remove $VV_PATH_TO_EXP\n";
	return 0;
    }
    return 1;
}
